import { Role } from './ad.types';
import { Target } from './candidate.types';
import { Actor } from './value-objects/actor.value-object';
import { Point } from './value-objects/point.value-object';
import { WayStep } from './value-objects/waystep.value-object';

export class CarpoolPathCreator {
  constructor(
    private readonly driverWaypoints: Point[],
    private readonly passengerWaypoints: Point[],
  ) {}

  public createCarpoolPath = (): WayStep[] => {
    const passengerWaysteps: WayStep[] = this._createPassengerWaysteps();
    if (this.driverWaypoints.length == 2) {
    }
    return passengerWaysteps;
  };

  private _createDriverWaysteps = (): WayStep[] =>
    this.driverWaypoints.map(
      (waypoint: Point, index: number) =>
        new WayStep({
          lon: waypoint.lon,
          lat: waypoint.lat,
          actors: [
            new Actor({
              role: Role.DRIVER,
              target: this._getTarget(index, this.driverWaypoints),
            }),
          ],
        }),
    );

  private _createPassengerWaysteps = (): WayStep[] => {
    const waysteps: WayStep[] = [];
    this.passengerWaypoints.forEach(
      (passengerWaypoint: Point, index: number) => {
        const waystep: WayStep = new WayStep({
          lon: passengerWaypoint.lon,
          lat: passengerWaypoint.lat,
          actors: [
            new Actor({
              role: Role.PASSENGER,
              target: this._getTarget(index, this.passengerWaypoints),
            }),
          ],
        });
        if (
          this.driverWaypoints.filter((driverWaypoint: Point) =>
            this._isSamePoint(driverWaypoint, passengerWaypoint),
          ).length > 0
        ) {
          waystep.actors.push(
            new Actor({
              role: Role.DRIVER,
              target: Target.NEUTRAL,
            }),
          );
        }
        waysteps.push(waystep);
      },
    );
    return waysteps;
  };

  private _isSamePoint = (point1: Point, point2: Point): boolean =>
    point1.lon === point2.lon && point1.lat === point2.lat;

  private _getTarget = (index: number, waypoints: Point[]): Target =>
    index == 0
      ? Target.START
      : index == waypoints.length - 1
      ? Target.FINISH
      : Target.INTERMEDIATE;
}
